@@ -35,6 +35,8 @@ import org.dmg.pmml.Field;
 import org.dmg.pmml.FieldName;
 import org.dmg.pmml.Header;
 import org.dmg.pmml.OpType;
+import org.dmg.pmml.Output;
+import org.dmg.pmml.OutputField;
 import org.dmg.pmml.PMML;
 import org.dmg.pmml.TransformationDictionary;
 
@@ -50,7 +52,7 @@ public class PMMLEncoder {
 	public PMML encodePMML(){
 
 		if(!Collections.disjoint(this.dataFields.keySet(), this.derivedFields.keySet())){
-			throw new IllegalArgumentException();
+			throw new IllegalStateException();
 		}
 
 		List<DataField> dataFields = new ArrayList<>(this.dataFields.values());
@@ -95,9 +97,7 @@ public class PMMLEncoder {
 	}
 
 	public void addDataField(DataField dataField){
-		FieldName name = dataField.getName();
-
-		checkName(name);
+		FieldName name = checkName(dataField);
 
 		this.dataFields.put(name, dataField);
 	}
@@ -122,7 +122,7 @@ public class PMMLEncoder {
 		DataField dataField = this.dataFields.remove(name);
 
 		if(dataField == null){
-			throw new IllegalArgumentException(name.getValue());
+			throw new IllegalArgumentException("Field " + name.getValue() + " is undefined");
 		}
 
 		return dataField;
@@ -145,17 +145,21 @@ public class PMMLEncoder {
 	}
 
 	public void addDerivedField(DerivedField derivedField){
-		FieldName name = derivedField.getName();
-
-		checkName(name);
+		FieldName name = checkName(derivedField);
 
 		this.derivedFields.put(name, derivedField);
 	}
 
 	public DerivedField createDerivedField(FieldName name, OpType opType, DataType dataType, Expression expression){
-		DerivedField derivedField = new DerivedField(opType, dataType)
-			.setName(name)
-			.setExpression(expression);
+		DerivedField derivedField = new DerivedField(name, opType, dataType, expression);
+
+		addDerivedField(derivedField);
+
+		return derivedField;
+	}
+
+	public DerivedOutputField createDerivedField(Output output, OutputField outputField){
+		DerivedOutputField derivedField = new DerivedOutputField(output, outputField);
 
 		addDerivedField(derivedField);
 
@@ -166,7 +170,7 @@ public class PMMLEncoder {
 		DerivedField derivedField = this.derivedFields.remove(name);
 
 		if(derivedField == null){
-			throw new IllegalArgumentException(name.getValue());
+			throw new IllegalArgumentException("Field " + name.getValue() + " is undefined");
 		}
 
 		return derivedField;
@@ -176,6 +180,10 @@ public class PMMLEncoder {
 		DataField dataField = getDataField(name);
 		DerivedField derivedField = getDerivedField(name);
 
+		if(dataField != null && derivedField != null){
+			throw new IllegalStateException();
+		} // End if
+
 		if(dataField != null && derivedField == null){
 			return dataField;
 		} else
@@ -184,7 +192,7 @@ public class PMMLEncoder {
 			return derivedField;
 		}
 
-		throw new IllegalArgumentException(name.getValue());
+		throw new IllegalArgumentException("Field " + name.getValue() + " is undefined");
 	}
 
 	public Field<?> toContinuous(FieldName name){
@@ -260,14 +268,17 @@ public class PMMLEncoder {
 		return this.defineFunctions;
 	}
 
-	private void checkName(FieldName name){
+	private FieldName checkName(Field<?> field){
+		FieldName name = field.getName();
 
 		if(name == null){
-			throw new NullPointerException();
+			throw new IllegalArgumentException();
 		} // End if
 
 		if(this.dataFields.containsKey(name) || this.derivedFields.containsKey(name)){
-			throw new IllegalArgumentException(name.getValue());
+			throw new IllegalArgumentException("Field " + name.getValue() + " is already defined");
 		}
+
+		return name;
 	}
 }
\ No newline at end of file
