@@ -19,8 +19,6 @@
 package org.jpmml.xgboost;
 
 import java.io.IOException;
-import java.util.ArrayList;
-import java.util.List;
 
 import org.dmg.pmml.DataType;
 import org.dmg.pmml.MathContext;
@@ -38,7 +36,7 @@ import org.jpmml.converter.ModelUtil;
 import org.jpmml.converter.PredicateManager;
 import org.jpmml.converter.Schema;
 
-public class RegTree {
+public class RegTree implements Loadable {
 
 	private int num_roots;
 
@@ -52,14 +50,15 @@ public class RegTree {
 
 	private int size_leaf_vector;
 
-	private List<Node> nodes;
+	private Node[] nodes;
 
-	private List<NodeStat> stats;
+	private NodeStat[] stats;
 
 
 	public RegTree(){
 	}
 
+	@Override
 	public void load(XGBoostDataInput input) throws IOException {
 		this.num_roots = input.readInt();
 		this.num_nodes = input.readInt();
@@ -70,27 +69,12 @@ public class RegTree {
 
 		input.readReserved(31);
 
-		this.nodes = new ArrayList<>();
-
-		for(int i = 0; i < this.num_nodes; i++){
-			Node node = new Node();
-			node.load(input);
-
-			this.nodes.add(node);
-		}
-
-		this.stats = new ArrayList<>();
-
-		for(int i = 0; i < this.num_nodes; i++){
-			NodeStat stat = new NodeStat();
-			stat.load(input);
-
-			this.stats.add(stat);
-		}
+		this.nodes = input.readObjectArray(Node.class, this.num_nodes);
+		this.stats = input.readObjectArray(NodeStat.class, this.num_nodes);
 	}
 
 	public TreeModel encodeTreeModel(PredicateManager predicateManager, Schema schema){
-		org.dmg.pmml.tree.Node root = encodeNode(new True(), predicateManager, 0, schema);
+		org.dmg.pmml.tree.Node root = encodeNode(True.INSTANCE, predicateManager, 0, schema);
 
 		TreeModel treeModel = new TreeModel(MiningFunction.REGRESSION, ModelUtil.createMiningSchema(schema.getLabel()), root)
 			.setSplitCharacteristic(TreeModel.SplitCharacteristic.BINARY_SPLIT)
@@ -101,9 +85,9 @@ public class RegTree {
 	}
 
 	private org.dmg.pmml.tree.Node encodeNode(Predicate predicate, PredicateManager predicateManager, int index, Schema schema){
-		Integer id = Integer.valueOf(index + 1);
+		Integer id = Integer.valueOf(index);
 
-		Node node = this.nodes.get(index);
+		Node node = this.nodes[index];
 
 		if(!node.is_leaf()){
 			int splitIndex = node.split_index();
@@ -151,11 +135,9 @@ public class RegTree {
 			org.dmg.pmml.tree.Node leftChild = encodeNode(leftPredicate, predicateManager, node.cleft(), schema);
 			org.dmg.pmml.tree.Node rightChild = encodeNode(rightPredicate, predicateManager, node.cright(), schema);
 
-			org.dmg.pmml.tree.Node result = new BranchNode()
+			org.dmg.pmml.tree.Node result = new BranchNode(null, predicate)
 				.setId(id)
-				.setScore(null) // XXX
 				.setDefaultChild(defaultLeft ? leftChild.getId() : rightChild.getId())
-				.setPredicate(predicate)
 				.addNodes(leftChild, rightChild);
 
 			return result;
@@ -164,16 +146,10 @@ public class RegTree {
 		{
 			Float value = node.leaf_value();
 
-			org.dmg.pmml.tree.Node result = new LeafNode()
-				.setId(id)
-				.setScore(value)
-				.setPredicate(predicate);
+			org.dmg.pmml.tree.Node result = new LeafNode(value, predicate)
+				.setId(id);
 
 			return result;
 		}
 	}
-
-	public List<Node> getNodes(){
-		return this.nodes;
-	}
 }
\ No newline at end of file
